"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CameraPublisher = void 0;
const IBackgroundEngine_1 = require("./background/interfaces/IBackgroundEngine");
const BackgroundEngine_1 = require("./background/BackgroundEngine");
const Channel_1 = require("./Channel");
const IChannelInterfaces_1 = require("./interfaces/IChannelInterfaces");
const IPublisherInterfaces_1 = require("./interfaces/IPublisherInterfaces");
const VolumeDetector_1 = require("./workers/VolumeDetector");
class CameraPublisher extends Channel_1.Channel {
    constructor(id, loggerLevel) {
        super(id, loggerLevel);
        this.PREFERRED_VIDEO_HEIGHT = 480;
        this.PREFERRED_VIDEO_WIDTH = 854;
        // Handle the need of a new negotiation
        // Occurs when conn.addTransceiver is called and when conn status changes
        this.conn.onnegotiationneeded = (event) => __awaiter(this, void 0, void 0, function* () { return yield this.onNegotiationNeeded(event); });
        this.initializeBackgroundEngine();
        this.logger.debug('Camera publisher created');
    }
    get isAudioMuted() {
        var _a;
        return !((_a = this.stream.getAudioTracks()[0]) === null || _a === void 0 ? void 0 : _a.enabled);
    }
    get isVideoHidden() {
        var _a;
        return !((_a = this.stream.getVideoTracks()[0]) === null || _a === void 0 ? void 0 : _a.enabled);
    }
    onNegotiationNeeded(event) {
        return __awaiter(this, void 0, void 0, function* () {
            this.emitter.next({
                type: IChannelInterfaces_1.EmittedDataType.CameraPublisherOffer,
                data: {
                    id: this.id,
                    offer: yield this.createOffer(),
                    isAudioMuted: this.isAudioMuted,
                    isVideoHidden: this.isVideoHidden,
                }
            });
        });
    }
    initializeBackgroundEngine() {
        this._backgroundEngine = new BackgroundEngine_1.BackgroundEngine(this.stream);
        this._backgroundEmitterSubscription = this._backgroundEngine.emitter.subscribe((event) => __awaiter(this, void 0, void 0, function* () {
            switch (event.type) {
                case IBackgroundEngine_1.BackgroundEventType.BlurStarted:
                    yield this.onBlurStarted(event.data);
                    break;
                case IBackgroundEngine_1.BackgroundEventType.BlurStopped:
                    yield this.onBlurStopped();
                    break;
                case IBackgroundEngine_1.BackgroundEventType.MaskStarted:
                    yield this.onMaskStarted(event.data.track, event.data.metadata);
                    break;
                case IBackgroundEngine_1.BackgroundEventType.MaskStopped:
                    yield this.onMaskStopped();
                    break;
                default:
                    throw new Error('Background engine event not recognized');
            }
        }));
    }
    onBlurStarted(blurredTrack) {
        return __awaiter(this, void 0, void 0, function* () {
            const videoSender = this.conn.getSenders().find(s => s.track.kind === 'video');
            if (videoSender) {
                yield this.replaceRTCPeerConnectionVideoTrack(blurredTrack, false);
            }
            else {
                this.addTransceiversToRTCPeerConnection();
            }
            this.emitter.next({
                type: IChannelInterfaces_1.EmittedDataType.BlurStarted,
                data: { publisherId: this.id }
            });
        });
    }
    onBlurStopped() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.replaceRTCPeerConnectionVideoTrack(this.stream.getVideoTracks()[0], false);
            this.emitter.next({
                type: IChannelInterfaces_1.EmittedDataType.BlurStopped,
                data: { publisherId: this.id }
            });
        });
    }
    onMaskStarted(maskedTrack, metadata) {
        return __awaiter(this, void 0, void 0, function* () {
            const videoSender = this.conn.getSenders().find(s => s.track.kind === 'video');
            if (videoSender) {
                yield this.replaceRTCPeerConnectionVideoTrack(maskedTrack, false);
            }
            else {
                this.addTransceiversToRTCPeerConnection();
            }
            this.emitter.next({
                type: IChannelInterfaces_1.EmittedDataType.MaskStarted,
                data: {
                    publisherId: this.id,
                    metadata: metadata
                }
            });
        });
    }
    onMaskStopped() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.replaceRTCPeerConnectionVideoTrack(this.stream.getVideoTracks()[0], false);
            this.emitter.next({
                type: IChannelInterfaces_1.EmittedDataType.MaskStopped,
                data: { publisherId: this.id }
            });
        });
    }
    dispose() {
        const _super = Object.create(null, {
            dispose: { get: () => super.dispose }
        });
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            this._backgroundEmitterSubscription.unsubscribe();
            this._backgroundEngine.dispose();
            (_a = this._volumeDetectorSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            yield ((_b = this._volumeDetector) === null || _b === void 0 ? void 0 : _b.dispose());
            yield _super.dispose.call(this);
        });
    }
    init(createAudioTrack, createVideoTrack, setup) {
        return __awaiter(this, void 0, void 0, function* () {
            const { audioDeviceId, muteAudio, videoDeviceId, hideVideo } = setup;
            const stream = yield this.getCameraStream(createAudioTrack, createVideoTrack, audioDeviceId, videoDeviceId);
            for (const track of stream === null || stream === void 0 ? void 0 : stream.getTracks()) {
                this.stream.addTrack(track);
                if (track.kind === 'audio') {
                    if (muteAudio === true) {
                        track.enabled = false;
                    }
                    if (VolumeDetector_1.VolumeDetector.isVolumeDetectorSupported) {
                        this._volumeDetector = new VolumeDetector_1.VolumeDetector(500, this.mediaStream);
                        this._volumeDetectorSubscription = this._volumeDetector.emitter$.subscribe((volume) => {
                            this.emitter.next({
                                type: IChannelInterfaces_1.EmittedDataType.Volume,
                                data: { channelId: this.id, volume: volume }
                            });
                        });
                        yield this._volumeDetector.start();
                    }
                    else {
                        console.log('This browser does not support volume detection feature.');
                    }
                }
                if (track.kind === 'video' && hideVideo === true) {
                    track.enabled = false;
                }
                if (track.kind === 'video' && setup.backgroundFilter) {
                    this.applyBackgroundFilter(setup.backgroundFilter);
                }
            }
            if (!setup.backgroundFilter) {
                this.addTransceiversToRTCPeerConnection();
            }
        });
    }
    applyBackgroundFilter(filter) {
        switch (filter.type) {
            case IPublisherInterfaces_1.BackgroundFilterType.Blur:
                this.applyBlur();
                break;
            case IPublisherInterfaces_1.BackgroundFilterType.Mask:
                this.applyMask(filter.imageUrl, filter.metadata);
                break;
            default:
                throw new Error('Background filter not recognized.');
        }
    }
    addTransceiversToRTCPeerConnection() {
        for (const track of this.stream.getTracks()) {
            const transceiver = this.conn.addTransceiver(track, {
                direction: 'sendonly'
            });
            this.logger.debug('Transceiver added to Publisher RTCPeerConnection', transceiver);
        }
    }
    getCameraStream(createAudioTrack, createVideoTrack, audioDeviceId, videoDeviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            const constraints = { audio: false, video: false };
            if (createAudioTrack) {
                constraints.audio = audioDeviceId ? { deviceId: audioDeviceId } : true;
            }
            if (createVideoTrack) {
                const videoConstraints = { width: this.PREFERRED_VIDEO_WIDTH, height: this.PREFERRED_VIDEO_HEIGHT };
                if (videoDeviceId) {
                    videoConstraints.deviceId = videoDeviceId;
                }
                constraints.video = videoConstraints;
            }
            return yield navigator.mediaDevices.getUserMedia(constraints);
        });
    }
    replaceAudioDevice(newDeviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            const stream = yield navigator.mediaDevices.getUserMedia({
                audio: {
                    deviceId: newDeviceId
                }
            });
            const track = stream.getAudioTracks()[0];
            // Replace stream track
            this.mediaStream.removeTrack(this.mediaStream.getAudioTracks()[0]);
            this.mediaStream.addTrack(track);
            // Replace RTCPeerConnection track
            const audioSender = this.conn.getSenders().find(s => s.track.kind === 'audio');
            audioSender.track.stop();
            audioSender.replaceTrack(track);
        });
    }
    replaceVideoDevice(newDeviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            const stream = yield navigator.mediaDevices.getUserMedia({
                video: {
                    deviceId: newDeviceId,
                    height: this.PREFERRED_VIDEO_HEIGHT,
                    width: this.PREFERRED_VIDEO_WIDTH,
                }
            });
            const track = stream.getVideoTracks()[0];
            this.replaceCameraVideoTrack(track);
            yield this.replaceRTCPeerConnectionVideoTrack(track, true);
        });
    }
    replaceCameraVideoTrack(track) {
        this.mediaStream.removeTrack(this.mediaStream.getVideoTracks()[0]);
        this.mediaStream.addTrack(track);
    }
    replaceRTCPeerConnectionVideoTrack(track, stopCurrentTrack) {
        return __awaiter(this, void 0, void 0, function* () {
            const videoSender = this.conn.getSenders().find(s => s.track.kind === 'video');
            if (stopCurrentTrack) {
                videoSender.track.stop();
            }
            yield videoSender.replaceTrack(track);
        });
    }
    toggleAudio() {
        const audioTrack = this.stream.getAudioTracks()[0];
        if (!audioTrack) {
            this.logger.warn('This publisher\'s stream doesn\'t have any audio track');
            return;
        }
        audioTrack.enabled = !audioTrack.enabled;
        this.emitter.next({
            type: IChannelInterfaces_1.EmittedDataType.AudioMutedChanged,
            data: {
                publisherId: this.id,
                muted: !audioTrack.enabled
            }
        });
    }
    toggleVideo() {
        const videoTrack = this.stream.getVideoTracks()[0];
        if (!videoTrack) {
            this.logger.warn('This publisher\'s stream doesn\'t have any video track');
            return;
        }
        videoTrack.enabled = !videoTrack.enabled;
        this.emitter.next({
            type: IChannelInterfaces_1.EmittedDataType.VideoHiddenChanged,
            data: {
                publisherId: this.id,
                hidden: !videoTrack.enabled
            }
        });
    }
    applyBlur() {
        this._backgroundEngine.applyBlur();
    }
    removeBlur() {
        this._backgroundEngine.removeBlur();
    }
    applyMask(imageUrl, metadata) {
        this._backgroundEngine.applyMask(imageUrl, metadata);
    }
    removeMask() {
        this._backgroundEngine.removeMask();
    }
}
exports.CameraPublisher = CameraPublisher;
