"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Channel = void 0;
const rxjs_1 = require("rxjs");
const IChannelInterfaces_1 = require("./interfaces/IChannelInterfaces");
const Logger_1 = require("./Logger");
class Channel {
    constructor(id, loggerLevel) {
        this._id = id;
        this.logger = new Logger_1.Logger(loggerLevel, id);
        // Initialize emitters
        this._emitter = new rxjs_1.Subject();
        // Initialize local RTCPeerConnection
        this.conn = new RTCPeerConnection();
        this.conn.onconnectionstatechange = (event) => this.onConnectionStateChange();
        // Handle new Ice candidates (send them to the remote peer)
        this.conn.onicecandidate = (event) => this.onIceCandidate(event);
        // Initialize the media stream
        this.mediaStream = new MediaStream();
    }
    get id() {
        return this._id;
    }
    get emitter() {
        return this._emitter;
    }
    get stream() {
        return this.mediaStream;
    }
    createOffer() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                // Generate SDP offer
                const offer = yield this.conn.createOffer();
                this.logger.debug('Setting local description: ', offer);
                // Set local RTCPeerConnection offer
                yield this.conn.setLocalDescription(offer);
                return offer.sdp;
            }
            catch (error) {
                this.logger.error(error);
            }
        });
    }
    dispose() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.stream) {
                for (const track of this.stream.getTracks()) {
                    track.stop();
                }
            }
            if (this.conn) {
                this.conn.close();
            }
        });
    }
    onConnectionStateChange() {
        this.logger.debug('RTCPeerConnection state changed: ', this.conn.iceConnectionState);
    }
    onIceCandidate(event) {
        const candidate = event.candidate;
        if (candidate) {
            // Send the new Ice candidate to the remote peer
            this._emitter.next({
                type: IChannelInterfaces_1.EmittedDataType.NewIceCandidate,
                data: {
                    channelId: this._id,
                    candidate: candidate
                }
            });
        }
        else {
            // End of negotiation: all Ice candidates have been sent
            this.logger.debug('All Ice Candidates have been sent');
        }
    }
    processIceCandidate(candidate) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug('Adding Ice Candidate to connection: ', candidate);
            yield this.conn.addIceCandidate(candidate);
        });
    }
    processAnswer(sdpAnswer) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.conn.setRemoteDescription({
                type: 'answer',
                sdp: sdpAnswer
            });
        });
    }
}
exports.Channel = Channel;
