"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Me = void 0;
const uuid_1 = require("uuid");
const rxjs_1 = require("rxjs");
const CameraPublisher_1 = require("./CameraPublisher");
const IMeInterfaces_1 = require("./interfaces/IMeInterfaces");
const Logger_1 = require("./Logger");
const ScreenPublisher_1 = require("./ScreenPublisher");
const IChannelInterfaces_1 = require("./interfaces/IChannelInterfaces");
class Me {
    constructor(config) {
        this._publishers = [];
        this._publishersSubscriptions = [];
        this._id = config.id,
            this._logger = new Logger_1.Logger(config.loggerLevel, config.id);
        this._nickname = config.nickname;
        this._logger.debug('Me created');
        this._emitter = new rxjs_1.Subject();
    }
    get emitter() {
        return this._emitter;
    }
    get id() {
        return this._id;
    }
    get nickname() {
        return this._nickname;
    }
    addCameraPublisher(createAudioTrack, createVideoTrack, initialSetup) {
        return __awaiter(this, void 0, void 0, function* () {
            const cameraPublisher = new CameraPublisher_1.CameraPublisher(`${this._id}_${(0, uuid_1.v4)()}`, this._logger.level);
            this.addAndSubscribePublisher(cameraPublisher);
            yield cameraPublisher.init(createAudioTrack, createVideoTrack, initialSetup);
            this._emitter.next({
                type: IMeInterfaces_1.EmittedDataType.NewCameraPublisher,
                data: {
                    id: cameraPublisher.id,
                    isAudioMuted: cameraPublisher.isAudioMuted,
                    isVideoHidden: cameraPublisher.isVideoHidden,
                    participantId: this._id,
                    stream: cameraPublisher.stream,
                }
            });
        });
    }
    addScreenPublisher() {
        return __awaiter(this, void 0, void 0, function* () {
            const screenPublisher = new ScreenPublisher_1.ScreenPublisher(`${this._id}_${(0, uuid_1.v4)()}`, this._logger.level);
            this.addAndSubscribePublisher(screenPublisher);
            yield screenPublisher.init();
            this._emitter.next({
                type: IMeInterfaces_1.EmittedDataType.NewScreenPublisher,
                data: {
                    id: screenPublisher.id,
                    participantId: this._id,
                    stream: screenPublisher.stream,
                }
            });
        });
    }
    addAndSubscribePublisher(publisher) {
        this._publishers.push(publisher);
        const subscription = publisher.emitter.subscribe((data) => this.onPublisherDataEmitted(data));
        this._publishersSubscriptions.push({
            publisherId: publisher.id,
            subscription: subscription
        });
    }
    onPublisherDataEmitted(data) {
        switch (data.type) {
            case IChannelInterfaces_1.EmittedDataType.AudioMutedChanged:
                this.onAudioMutedChanged(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.BlurStarted:
                this.onBlurStarted(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.BlurStopped:
                this.onBlurStopped(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.MaskStarted:
                this.onMaskStarted(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.MaskStopped:
                this.onMaskStopped(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.NewIceCandidate:
                this.onNewIceCandidate(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.CameraPublisherOffer:
                this.onCameraPublisherOffer(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.ScreenPublisherOffer:
                this.onScreenPublisherOffer(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.ScreenSharingStopped:
                this.onScreensharingStopped(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.VideoHiddenChanged:
                this.onVideoHiddenChanged(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.Volume:
                this.onVolume(data.data);
                break;
            default:
                this._logger.error('Received unknown data from publisher: ', data);
                break;
        }
    }
    onAudioMutedChanged(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.AudioMutedChanged,
            data: {
                publisherId: params.publisherId,
                muted: params.muted
            }
        });
    }
    onBlurStarted(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.BlurStarted,
            data: params
        });
    }
    onBlurStopped(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.BlurStopped,
            data: params
        });
    }
    onMaskStarted(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.MaskStarted,
            data: params
        });
    }
    onMaskStopped(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.MaskStopped,
            data: params
        });
    }
    onNewIceCandidate(params) {
        this._emitter.next({
            data: {
                candidate: params.candidate,
                publisherId: params.channelId,
                participantId: this._id
            },
            type: IMeInterfaces_1.EmittedDataType.NewIceCandidate
        });
    }
    onVideoHiddenChanged(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.VideoHiddenChanged,
            data: {
                publisherId: params.publisherId,
                hidden: params.hidden
            }
        });
    }
    onVolume(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.Volume,
            data: {
                publisherId: params.channelId,
                volume: params.volume
            }
        });
    }
    onCameraPublisherOffer(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.CameraPublisherOffer,
            data: params,
        });
    }
    onScreenPublisherOffer(params) {
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.ScreenPublisherOffer,
            data: params,
        });
    }
    onScreensharingStopped(params) {
        const publisher = this._publishers.find(c => c.id === params.publisherId);
        this.stopScreensharing(publisher);
    }
    processAnswer(publisherId, answer) {
        const publisher = this._publishers.find(c => c.id === publisherId);
        if (publisher) {
            publisher.processAnswer(answer);
        }
    }
    processIceCandidate(publisherId, candidate) {
        const publisher = this._publishers.find(c => c.id === publisherId);
        if (publisher) {
            publisher.processIceCandidate(candidate);
        }
    }
    stopScreensharing(screenPublisher) {
        const subscription = this._publishersSubscriptions.find(cs => cs.publisherId === screenPublisher.id);
        if (subscription) {
            subscription.subscription.unsubscribe();
            this._publishersSubscriptions.splice(this._publishersSubscriptions.indexOf(subscription), 1);
        }
        screenPublisher.dispose();
        this._publishers.splice(this._publishers.indexOf(screenPublisher), 1);
        this._emitter.next({
            type: IMeInterfaces_1.EmittedDataType.RemovePublisher,
            data: { id: screenPublisher.id }
        });
    }
    applyBlur() {
        const publisher = this._publishers.find(c => c instanceof CameraPublisher_1.CameraPublisher);
        publisher.applyBlur();
    }
    removeBlur() {
        const publisher = this._publishers.find(c => c instanceof CameraPublisher_1.CameraPublisher);
        publisher.removeBlur();
    }
    applyMask(imageUrl, metadata) {
        const publisher = this._publishers.find(c => c instanceof CameraPublisher_1.CameraPublisher);
        publisher.applyMask(imageUrl, metadata);
    }
    removeMask() {
        const publisher = this._publishers.find(c => c instanceof CameraPublisher_1.CameraPublisher);
        publisher.removeMask();
    }
    dispose() {
        for (const publisher of this._publishers) {
            publisher.dispose();
        }
        for (const publisher of this._publishersSubscriptions) {
            publisher.subscription.unsubscribe();
        }
    }
    replaceAudioTrack(publisherId, deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            const publisher = this._publishers.find(c => c.id === publisherId);
            if (publisher && publisher instanceof CameraPublisher_1.CameraPublisher) {
                publisher.replaceAudioDevice(deviceId);
            }
        });
    }
    replaceVideoTrack(publisherId, deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            const publisher = this._publishers.find(c => c.id === publisherId);
            if (publisher && publisher instanceof CameraPublisher_1.CameraPublisher) {
                publisher.replaceVideoDevice(deviceId);
            }
        });
    }
    toggleAudio(publisherId) {
        const publisher = this._publishers.find(c => c.id === publisherId);
        if (publisher && publisher instanceof CameraPublisher_1.CameraPublisher) {
            publisher.toggleAudio();
        }
    }
    toggleScreensharing() {
        return __awaiter(this, void 0, void 0, function* () {
            const publisher = this._publishers.find(c => c instanceof ScreenPublisher_1.ScreenPublisher);
            if (publisher) {
                this.stopScreensharing(publisher);
            }
            else {
                yield this.addScreenPublisher();
            }
        });
    }
    toggleVideo(publisherId) {
        const publisher = this._publishers.find(c => c.id === publisherId);
        if (publisher && publisher instanceof CameraPublisher_1.CameraPublisher) {
            publisher.toggleVideo();
        }
    }
}
exports.Me = Me;
