"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Participant = void 0;
const rxjs_1 = require("rxjs");
const CameraSubscriber_1 = require("./CameraSubscriber");
const IChannelInterfaces_1 = require("./interfaces/IChannelInterfaces");
const IParticipantInterfaces_1 = require("./interfaces/IParticipantInterfaces");
const Logger_1 = require("./Logger");
const ScreenSubscriber_1 = require("./ScreenSubscriber");
class Participant {
    constructor(config) {
        this._subscribers = [];
        this._subscribersSubscriptions = [];
        this._id = config.id,
            this._logger = new Logger_1.Logger(config.loggerLevel, config.id);
        this._nickname = config.nickname;
        this._logger.debug('Participant created');
        this._emitter = new rxjs_1.Subject();
    }
    get emitter() {
        return this._emitter;
    }
    get id() {
        return this._id;
    }
    get nickname() {
        return this._nickname;
    }
    addCameraSubscriber(id, receiveAudio, receiveVideo, isAudioMuted, isVideoHidden) {
        return __awaiter(this, void 0, void 0, function* () {
            const cameraSubscriber = new CameraSubscriber_1.CameraSubscriber(id, this._logger.level);
            yield cameraSubscriber.init(receiveAudio, receiveVideo);
            this.addAndSubscribeSubscriber(cameraSubscriber);
            yield cameraSubscriber.createAndEmitOffer();
            this._emitter.next({
                type: IParticipantInterfaces_1.EmittedDataType.NewCameraSubscriber,
                data: {
                    id: cameraSubscriber.id,
                    participantId: this._id,
                    stream: cameraSubscriber.stream,
                    isAudioMuted: isAudioMuted,
                    isVideoHidden: isVideoHidden,
                }
            });
        });
    }
    addScreenSubscriber(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const screenSubscriber = new ScreenSubscriber_1.ScreenSubscriber(id, this._logger.level);
            this.addAndSubscribeSubscriber(screenSubscriber);
            yield screenSubscriber.createAndEmitOffer();
            this._emitter.next({
                type: IParticipantInterfaces_1.EmittedDataType.NewScreenSubscriber,
                data: {
                    id: screenSubscriber.id,
                    participantId: this._id,
                    stream: screenSubscriber.stream,
                    isAudioMuted: false,
                    isVideoHidden: false,
                }
            });
        });
    }
    addAndSubscribeSubscriber(subscriber) {
        this._subscribers.push(subscriber);
        const subscription = subscriber.emitter.subscribe((data) => this.onSubscriberDataEmitted(data));
        this._subscribersSubscriptions.push({
            subscriberId: subscriber.id,
            subscription: subscription
        });
    }
    dispose() {
        for (const subscriber of this._subscribers) {
            subscriber.dispose();
        }
        for (const subscription of this._subscribersSubscriptions) {
            subscription.subscription.unsubscribe();
        }
    }
    onSubscriberDataEmitted(data) {
        switch (data.type) {
            case IChannelInterfaces_1.EmittedDataType.NewIceCandidate:
                this.onNewIceCandidate(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.SubscriberOffer:
                this.onOffer(data.data);
                break;
            case IChannelInterfaces_1.EmittedDataType.Volume:
                this.onVolume(data.data);
                break;
            default:
                this._logger.error('Received unknown data from subscriber: ', data);
                break;
        }
    }
    onNewIceCandidate(params) {
        this._emitter.next({
            data: {
                candidate: params.candidate,
                subscriberId: params.channelId,
                participantId: this._id
            },
            type: IParticipantInterfaces_1.EmittedDataType.NewIceCandidate
        });
    }
    onOffer(params) {
        this._emitter.next({
            data: {
                id: params.id,
                offer: params.offer,
                participantId: this._id
            },
            type: IParticipantInterfaces_1.EmittedDataType.SubscriberOffer
        });
    }
    onVolume(params) {
        this._emitter.next({
            type: IParticipantInterfaces_1.EmittedDataType.Volume,
            data: {
                participantId: this.id,
                subscriberId: params.channelId,
                volume: params.volume
            }
        });
    }
    processAnswer(subscriberId, answer) {
        const subscriber = this._subscribers.find(c => c.id === subscriberId);
        if (subscriber) {
            subscriber.processAnswer(answer);
        }
    }
    processIceCandidate(subscriberId, candidate) {
        const subscriber = this._subscribers.find(c => c.id === subscriberId);
        if (subscriber) {
            subscriber.processIceCandidate(candidate);
        }
    }
    removeSubscriber(id) {
        const subscription = this._subscribersSubscriptions.find(cs => cs.subscriberId === id);
        if (subscription) {
            subscription.subscription.unsubscribe();
            this._subscribersSubscriptions.splice(this._subscribersSubscriptions.indexOf(subscription), 1);
        }
        const subscriber = this._subscribers.find(c => c.id === id);
        if (subscriber) {
            subscriber.dispose();
            this._subscribers.splice(this._subscribers.indexOf(subscriber), 1);
        }
    }
}
exports.Participant = Participant;
