import { Subject } from 'rxjs';
import { IMeetingConfig, IMessageParams, ISendMessageResult, ISessionEmittedData } from './interfaces/ISessionInterfaces';
import { LoggerLevel } from "./Logger";
/**
 * The entity responsible for creating and managing WebRTC session.\
 * All the interactions with vidaoo-browser library go through this entity.
 */
export declare class Session {
    private _capabilities;
    private _emitter;
    private _fileBox;
    private _me;
    private _meSubscription;
    private _mode;
    private _nickname;
    private _logger;
    private _participants;
    private _participantsSubscriptions;
    private _initialSetup;
    private _sessionType;
    private _signaler;
    private _signalerSubscription;
    /**
     * [RxJS Subject](https://rxjs-dev.firebaseapp.com/guide/subject) responsible for sending data from the library to the application.
     */
    get emitter(): Subject<ISessionEmittedData>;
    constructor(loggerLevel?: LoggerLevel);
    abortFileUpload(fileId: string): void;
    /**
     * Deletes a message.
     */
    deleteMessage(id: string): void;
    /**
     * Disposes all internal resources of the library.\
     * This method should always be called at the end of the session.
     */
    dispose(): Promise<void>;
    /**
     * Edits a message.
     */
    editMessage(id: string, text: string, params?: IMessageParams): void;
    private init;
    /**
     * Initializes the session as a meeting.\
     * This method should always be called one time when joining a session.
     */
    joinMeeting(meetingConfig: IMeetingConfig): void;
    private onMeetingConnect;
    private onParticipantDataEmitted;
    private onParticipantNewIceCandidate;
    private onParticipantNewCameraSubscriber;
    private onParticipantNewScreenSubscriber;
    private onParticipantSubscriberOffer;
    private onParticipantVolume;
    private onSignalerDataEmitted;
    private onSignalerAnswer;
    private onSignalerConnect;
    private onSignalerError;
    private onSignalerDeleteMessage;
    private onSignalerEditMessage;
    private onSignalerFileChunk;
    private onSignalerFileProgress;
    private onSignalerNewIceCandidate;
    private onSignalerNewMessage;
    private onSignalerNewParticipant;
    private onSignalerNewPublisher;
    private onSignalerPublisherStatus;
    private onSignalerRecStarted;
    private onSignalerRecStopped;
    private onSignalerRemoveParticipant;
    private onSignalerRemovePublisher;
    private onSignalerSessionEnded;
    private onSignalerTyping;
    private onSignalerWelcome;
    private onMeDataEmitted;
    private onMeAudioMutedChanged;
    private onMeBlurStarted;
    private onMeBlurStopped;
    private onMeMaskStarted;
    private onMeMaskStopped;
    private onMeNewCameraPublisher;
    private onMeNewIceCandidate;
    private onMeNewScreenPublisher;
    private onMeCameraPublisherOffer;
    private onMeScreenPublisherOffer;
    private onMeRemovePublisher;
    private onMeVideoHiddenChanged;
    private onMeVolume;
    /**
     * Replaces own audio track by specifying a different device (microphone) Id.\
     * \
     * See also: [MediaDeviceInfo](https://developer.mozilla.org/en-US/docs/Web/API/MediaDeviceInfo)
     */
    replaceAudioTrack(publisherId: string, deviceId: string): Promise<void>;
    /**
    * Replaces own video track by specifying a different device (camera) Id.\
    * \
    * See also: [MediaDeviceInfo](https://developer.mozilla.org/en-US/docs/Web/API/MediaDeviceInfo)
    */
    replaceVideoTrack(publisherId: string, deviceId: string): Promise<void>;
    /**
     * Sends a message to session participants.
     */
    sendMessage(text: string, params?: IMessageParams): Promise<ISendMessageResult>;
    /**
     * Informs other participants if you are typing or not.
     */
    sendTyping(typing: boolean): void;
    /**
   * Apply blur background effect to own video track.
   */
    applyBlur(): void;
    /**
   * Remove blur background effect to own video track.
   */
    removeBlur(): void;
    /**
   * Apply mask effect (change background with a custom image) to own video track.
     * @param imageUrl The URL of the image to use as a background. If the URL domain of the image is different from the web application domain, the server providing the image must add the following response header "Access-Control-Allow-Origin: *".
     * @param metadata Data that will return with the [[MaskStarted]] event. Useful, for example, to keep track of the image chosen as a background.
   */
    applyMask(imageUrl: string, metadata?: any): void;
    /**
   * Remove mask effect from own video track.
   */
    removeMask(): void;
    /**
     * Toggles screensharing.
     */
    toggleScreensharing(): Promise<void>;
    /**
     * Toggles audio (microphone).\
     * This can be used to mute/unmute own microphone.
     */
    toggleAudio(publisherId: string): void;
    /**
     * Toggles video (camera).\
     * This can be used to show/hide own camera.
     */
    toggleVideo(publisherId: string): void;
}
