"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Session = void 0;
const rxjs_1 = require("rxjs");
const IMeInterfaces_1 = require("./interfaces/IMeInterfaces");
const IParticipantInterfaces_1 = require("./interfaces/IParticipantInterfaces");
const ISignaler_1 = require("./interfaces/ISignaler");
const ErrorCode_1 = require("./enums/ErrorCode");
const Logger_1 = require("./Logger");
const Participant_1 = require("./Participant");
const SocketIOSignaler_1 = require("./SocketIOSignaler");
const Mode_1 = require("./enums/Mode");
const FileBox_1 = require("./FileBox");
const SessionType_1 = require("./enums/SessionType");
const SessionEmittedDataType_1 = require("./enums/SessionEmittedDataType");
const uuid_1 = require("uuid");
const StreamKind_1 = require("./enums/StreamKind");
const Me_1 = require("./Me");
/**
 * The entity responsible for creating and managing WebRTC session.\
 * All the interactions with vidaoo-browser library go through this entity.
 */
class Session {
    constructor(loggerLevel = Logger_1.LoggerLevel.Warn) {
        this._participants = [];
        this._participantsSubscriptions = [];
        this._sessionType = SessionType_1.SessionType.Unset;
        this._logger = new Logger_1.Logger(loggerLevel);
        this._emitter = new rxjs_1.Subject();
        this._fileBox = new FileBox_1.FileBox(loggerLevel);
    }
    /**
     * [RxJS Subject](https://rxjs-dev.firebaseapp.com/guide/subject) responsible for sending data from the library to the application.
     */
    get emitter() {
        return this._emitter;
    }
    abortFileUpload(fileId) {
        if (this._fileBox.some(fileId)) {
            this._signaler.send('file-abort', { id: fileId });
            this._fileBox.remove(fileId);
        }
    }
    /**
     * Deletes a message.
     */
    deleteMessage(id) {
        this._signaler.send('delete-message', { id: id });
    }
    /**
     * Disposes all internal resources of the library.\
     * This method should always be called at the end of the session.
     */
    dispose() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('Disposing session object');
            (_a = this._meSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            (_b = this._me) === null || _b === void 0 ? void 0 : _b.dispose();
            for (const participantSubscription of this._participantsSubscriptions) {
                participantSubscription.subscription.unsubscribe();
            }
            this._participantsSubscriptions.splice(0, this._participantsSubscriptions.length);
            for (const participant of this._participants) {
                participant.dispose();
            }
            this._participants.splice(0, this._participants.length);
            this._signalerSubscription.unsubscribe();
            this._signaler.dispose();
            this._sessionType = SessionType_1.SessionType.Unset;
        });
    }
    /**
     * Edits a message.
     */
    editMessage(id, text, params) {
        const message = {
            id: id,
            text: text
        };
        if (params === null || params === void 0 ? void 0 : params.mentions) {
            message.mentions = params.mentions;
        }
        this._signaler.send('edit-message', message);
    }
    init(webSocketUrl, sessionId, nickname) {
        this._signaler = new SocketIOSignaler_1.SocketIOSignaler(webSocketUrl, sessionId, nickname);
    }
    /**
     * Initializes the session as a meeting.\
     * This method should always be called one time when joining a session.
     */
    joinMeeting(meetingConfig) {
        try {
            if (this._sessionType !== SessionType_1.SessionType.Unset) {
                throw new Error('This session has already been initialized');
            }
            this._sessionType = SessionType_1.SessionType.Meeting;
            this._mode = Object.values(Mode_1.Mode).includes(meetingConfig.mode) ? meetingConfig.mode : Mode_1.Mode.AudioAndVideo;
            this._nickname = meetingConfig.nickname;
            this.init(meetingConfig.webSocketUrl, meetingConfig.meetingId, meetingConfig.nickname);
            this._signalerSubscription = this._signaler.emitter.subscribe((data) => __awaiter(this, void 0, void 0, function* () { return yield this.onSignalerDataEmitted(data); }));
            this._initialSetup = meetingConfig.publisherInitialSetup;
        }
        catch (error) {
            this._logger.error(error);
        }
    }
    onMeetingConnect() {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug(`Websocket connected. Socket Id: ${this._signaler.id}`);
            this._emitter.next({
                type: SessionEmittedDataType_1.SessionEmittedDataType.WebSocketConnected,
                data: {
                    id: this._signaler.id
                }
            });
        });
    }
    onParticipantDataEmitted(data) {
        switch (data.type) {
            case IParticipantInterfaces_1.EmittedDataType.NewIceCandidate:
                this.onParticipantNewIceCandidate(data.data);
                break;
            case IParticipantInterfaces_1.EmittedDataType.NewCameraSubscriber:
                this.onParticipantNewCameraSubscriber(data.data);
                break;
            case IParticipantInterfaces_1.EmittedDataType.NewScreenSubscriber:
                this.onParticipantNewScreenSubscriber(data.data);
                break;
            case IParticipantInterfaces_1.EmittedDataType.SubscriberOffer:
                this.onParticipantSubscriberOffer(data.data);
                break;
            case IParticipantInterfaces_1.EmittedDataType.Volume:
                this.onParticipantVolume(data.data);
                break;
            default:
                this._logger.error('Received unknown data from participant: ', data);
                break;
        }
    }
    onParticipantNewIceCandidate(params) {
        const newIceCandidate = {
            id: params.subscriberId,
            candidate: params.candidate
        };
        this._logger.debug('Sending subscriber new-ice-candidate: ', newIceCandidate);
        this._signaler.send('new-ice-candidate', newIceCandidate);
    }
    onParticipantNewCameraSubscriber(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.NewSubscriber,
            data: {
                id: params.id,
                streamKind: StreamKind_1.StreamKind.Camera,
                stream: params.stream,
                isAudioMuted: params.isAudioMuted,
                isVideoHidden: params.isVideoHidden,
                participantId: params.participantId,
            }
        });
    }
    onParticipantNewScreenSubscriber(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.NewSubscriber,
            data: {
                id: params.id,
                streamKind: StreamKind_1.StreamKind.Screen,
                stream: params.stream,
                participantId: params.participantId,
            }
        });
    }
    onParticipantSubscriberOffer(params) {
        this._logger.debug('Sending subscribe: ', params);
        this._signaler.send('subscribe', params);
    }
    onParticipantVolume(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.ParticipantVolume,
            data: params
        });
    }
    onSignalerDataEmitted(data) {
        return __awaiter(this, void 0, void 0, function* () {
            switch (data.type) {
                case ISignaler_1.SignalType.Answer:
                    yield this.onSignalerAnswer(data.params);
                    break;
                case ISignaler_1.SignalType.Connect:
                    yield this.onSignalerConnect();
                    break;
                case ISignaler_1.SignalType.DeleteMessage:
                    yield this.onSignalerDeleteMessage(data.params);
                    break;
                case ISignaler_1.SignalType.EditMessage:
                    yield this.onSignalerEditMessage(data.params);
                    break;
                case ISignaler_1.SignalType.Error:
                    yield this.onSignalerError(data.params);
                    break;
                case ISignaler_1.SignalType.FileChunk:
                    yield this.onSignalerFileChunk(data.params);
                    break;
                case ISignaler_1.SignalType.FileProgress:
                    yield this.onSignalerFileProgress(data.params);
                    break;
                case ISignaler_1.SignalType.NewIceCandidate:
                    yield this.onSignalerNewIceCandidate(data.params);
                    break;
                case ISignaler_1.SignalType.NewMessage:
                    yield this.onSignalerNewMessage(data.params);
                    break;
                case ISignaler_1.SignalType.NewParticipant:
                    yield this.onSignalerNewParticipant(data.params);
                    break;
                case ISignaler_1.SignalType.NewPublisher:
                    yield this.onSignalerNewPublisher(data.params);
                    break;
                case ISignaler_1.SignalType.PublisherStatus:
                    yield this.onSignalerPublisherStatus(data.params);
                    break;
                case ISignaler_1.SignalType.RecStarted:
                    yield this.onSignalerRecStarted(data.params);
                    break;
                case ISignaler_1.SignalType.RecStopped:
                    yield this.onSignalerRecStopped();
                    break;
                case ISignaler_1.SignalType.RemoveParticipant:
                    yield this.onSignalerRemoveParticipant(data.params);
                    break;
                case ISignaler_1.SignalType.RemovePublisher:
                    yield this.onSignalerRemovePublisher(data.params);
                    break;
                case ISignaler_1.SignalType.SessionEnded:
                    yield this.onSignalerSessionEnded(data.params);
                    break;
                case ISignaler_1.SignalType.Typing:
                    yield this.onSignalerTyping(data.params);
                    break;
                case ISignaler_1.SignalType.Welcome:
                    yield this.onSignalerWelcome(data.params);
                    break;
                default:
                    this._logger.error('Received unknown data from signaler: ', data);
                    break;
            }
        });
    }
    onSignalerAnswer(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._me.id === params.participantId) {
                this._logger.debug('answer received for publisher: ', params);
                yield this._me.processAnswer(params.id, params.answer);
            }
            else {
                this._logger.debug('answer received for subscriber: ', params);
                const participant = this._participants.find(p => p.id === params.participantId);
                if (participant) {
                    yield participant.processAnswer(params.id, params.answer);
                }
            }
        });
    }
    onSignalerConnect() {
        return __awaiter(this, void 0, void 0, function* () {
            switch (this._sessionType) {
                case SessionType_1.SessionType.Meeting:
                    yield this.onMeetingConnect();
                    break;
                case SessionType_1.SessionType.Webinar:
                    //await this.onWebinarConnect();
                    break;
                default:
                    this._logger.error('Session not yet initialized');
                    break;
            }
        });
    }
    onSignalerError(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.info('error-message received: ', params);
            switch (params.code) {
                case ErrorCode_1.ErrorCode.Publish:
                    this._logger.warn('Your publish offer is not compatible with this session. Please check session capabilities.');
                    break;
                case ErrorCode_1.ErrorCode.Subscribe:
                    this._logger.warn('Your subscribe offer is not compatible with this session. Please check session and new-publisher capabilities.');
                    break;
                case ErrorCode_1.ErrorCode.UploadFile:
                    const messageId = params.metadata.messageId;
                    this._fileBox.removeByMessage(messageId);
                    break;
            }
            this._emitter.next({ type: SessionEmittedDataType_1.SessionEmittedDataType.Error, data: params });
        });
    }
    onSignalerDeleteMessage(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.info('delete-message received: ', params);
            this._emitter.next({ type: SessionEmittedDataType_1.SessionEmittedDataType.DeleteMessage, data: params });
        });
    }
    onSignalerEditMessage(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.info('edit-message received: ', params);
            this._emitter.next({ type: SessionEmittedDataType_1.SessionEmittedDataType.EditMessage, data: params });
        });
    }
    onSignalerFileChunk(params) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this._logger.verbose('file-chunk received: ', params);
                const chunk = this._fileBox.getChunk(params.id, params.chunkNumber);
                this._signaler.send('file-chunk', { id: params.id, chunk: chunk });
            }
            catch (error) {
                this._logger.error(error);
            }
        });
    }
    onSignalerFileProgress(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('file-progress received: ', params);
            this._emitter.next({ type: SessionEmittedDataType_1.SessionEmittedDataType.FileProgress, data: params });
        });
    }
    onSignalerNewIceCandidate(params) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._me.id === params.participantId) {
                this._logger.debug('new-ice-candidate received for publisher: ', params);
                this._me.processIceCandidate(params.id, params.candidate);
            }
            else {
                this._logger.debug('new-ice-candidate received for subscriber: ', params);
                const participant = this._participants.find(p => p.id === params.participantId);
                if (participant) {
                    participant.processIceCandidate(params.id, params.candidate);
                }
            }
        });
    }
    onSignalerNewMessage(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('new-message received: ', params);
            const newMessage = {
                id: params.id,
                participantId: params.participantId,
                nickname: params.nickname,
                sentAt: new Date(params.sentAt)
            };
            if (params.text) {
                newMessage.text = params.text;
            }
            if (params.attachments) {
                newMessage.attachments = params.attachments;
            }
            if (params.replyId) {
                newMessage.replyId = params.replyId;
            }
            if (params.mentions) {
                newMessage.mentions = params.mentions;
            }
            this._emitter.next({ type: SessionEmittedDataType_1.SessionEmittedDataType.NewMessage, data: newMessage });
        });
    }
    onSignalerNewParticipant(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('new-participant received: ', params);
            const participant = new Participant_1.Participant({
                id: params.id,
                loggerLevel: this._logger.level,
                nickname: params.nickname
            });
            const subscription = participant.emitter.subscribe((data) => this.onParticipantDataEmitted(data));
            this._participantsSubscriptions.push({ participantId: participant.id, subscription: subscription });
            this._participants.push(participant);
            this._emitter.next({
                type: SessionEmittedDataType_1.SessionEmittedDataType.NewParticipant,
                data: {
                    id: participant.id,
                    nickname: participant.nickname
                }
            });
        });
    }
    onSignalerNewPublisher(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('new-publisher received: ', params);
            const participant = this._participants.find(p => p.id === params.participantId);
            if (!participant) {
                return;
            }
            if (this._mode === Mode_1.Mode.AudioOnly && !params.isAudioPresent || this._mode === Mode_1.Mode.VideoOnly && !params.isVideoPresent) {
                this._logger.warn(`Skipping new-publisher due to incompatible mode. My mode: ${this._mode}, publisher audio: ${params.isAudioPresent}, publisher video: ${params.isVideoPresent}`);
                return;
            }
            if (params.streamKind === StreamKind_1.StreamKind.Camera) {
                yield participant.addCameraSubscriber(params.id, params.isAudioPresent && (this._mode === Mode_1.Mode.AudioAndVideo || this._mode === Mode_1.Mode.AudioOnly), params.isVideoPresent && (this._mode === Mode_1.Mode.AudioAndVideo || this._mode === Mode_1.Mode.VideoOnly), params.isAudioMuted, params.isVideoHidden);
            }
            else if (params.streamKind === StreamKind_1.StreamKind.Screen) {
                yield participant.addScreenSubscriber(params.id);
            }
        });
    }
    onSignalerPublisherStatus(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('publisher-status received: ', params);
            this._emitter.next({
                type: SessionEmittedDataType_1.SessionEmittedDataType.SubscriberStatus,
                data: {
                    id: params.id,
                    participantId: params.participantId,
                    isAudioMuted: params.isAudioMuted,
                    isVideoHidden: params.isVideoHidden
                }
            });
        });
    }
    onSignalerRecStarted(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('rec-started received: ', params);
            this._emitter.next({
                type: SessionEmittedDataType_1.SessionEmittedDataType.RecStarted,
                data: {
                    id: params.id
                }
            });
        });
    }
    onSignalerRecStopped() {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('rec-stopped received');
            this._emitter.next({
                type: SessionEmittedDataType_1.SessionEmittedDataType.RecStopped
            });
        });
    }
    onSignalerRemoveParticipant(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('remove-participant received: ', params);
            const participantSubscription = this._participantsSubscriptions.find(ps => ps.participantId === params.id);
            if (participantSubscription) {
                participantSubscription.subscription.unsubscribe();
                this._participantsSubscriptions.splice(this._participantsSubscriptions.indexOf(participantSubscription), 1);
            }
            const participant = this._participants.find(p => p.id === params.id);
            if (participant) {
                yield participant.dispose();
                this._participants.splice(this._participants.indexOf(participant), 1);
                this._emitter.next({
                    type: SessionEmittedDataType_1.SessionEmittedDataType.RemoveParticipant,
                    data: {
                        id: params.id
                    }
                });
            }
        });
    }
    onSignalerRemovePublisher(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('remove-publisher received: ', params);
            const participant = this._participants.find(p => p.id === params.participantId);
            if (participant) {
                yield participant.removeSubscriber(params.id);
                this._emitter.next({
                    type: SessionEmittedDataType_1.SessionEmittedDataType.RemoveSubscriber,
                    data: {
                        id: params.id,
                        participantId: params.participantId
                    }
                });
            }
        });
    }
    onSignalerSessionEnded(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('session-ended received: ', params);
            this._emitter.next({ type: SessionEmittedDataType_1.SessionEmittedDataType.SessionEnded, data: params });
        });
    }
    onSignalerTyping(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('typing received: ', params);
            this._emitter.next({ type: SessionEmittedDataType_1.SessionEmittedDataType.Typing, data: params });
        });
    }
    onSignalerWelcome(params) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            this._logger.debug('welcome received: ', params);
            this._capabilities = params.capabilities;
            this._emitter.next({
                type: SessionEmittedDataType_1.SessionEmittedDataType.Welcome,
                data: {
                    participantId: params.participantId,
                    id: params.id,
                    topic: params.topic,
                    capabilities: params.capabilities,
                    recordingId: params.recordingId,
                    feedback: params.feedback,
                    guestRedirectionUrl: params.guestRedirectionUrl
                }
            });
            this._me = new Me_1.Me({
                id: params.participantId,
                loggerLevel: this._logger.level,
                nickname: this._nickname
            });
            this._meSubscription = this._me.emitter.subscribe((data) => __awaiter(this, void 0, void 0, function* () { return yield this.onMeDataEmitted(data); }));
            delete this._nickname;
            this._emitter.next({
                type: SessionEmittedDataType_1.SessionEmittedDataType.Me,
                data: { id: this._me.id }
            });
            const devices = yield navigator.mediaDevices.enumerateDevices();
            const isCameraPresent = devices.some(d => d.kind === 'videoinput');
            const isMicrophonePresent = devices.some(d => d.kind === 'audioinput');
            if (isCameraPresent || isMicrophonePresent) {
                if ((_a = this._initialSetup) === null || _a === void 0 ? void 0 : _a.audioDeviceId) {
                    if (!devices.some(d => d.deviceId === this._initialSetup.audioDeviceId)) {
                        console.warn('Selected microphone not found.');
                        this._initialSetup.audioDeviceId = null;
                    }
                }
                if ((_b = this._initialSetup) === null || _b === void 0 ? void 0 : _b.videoDeviceId) {
                    if (!devices.some(d => d.deviceId === this._initialSetup.videoDeviceId)) {
                        console.warn('Selected camera not found.');
                        this._initialSetup.videoDeviceId = null;
                    }
                }
                yield this._me.addCameraPublisher(isMicrophonePresent && this._capabilities.audio && (this._mode === Mode_1.Mode.AudioAndVideo || this._mode === Mode_1.Mode.AudioOnly), isCameraPresent && this._capabilities.video && (this._mode === Mode_1.Mode.AudioAndVideo || this._mode === Mode_1.Mode.VideoOnly), this._initialSetup);
            }
            else {
                this._logger.info('No audio/video devices found. Publisher creation skipped.');
            }
            delete this._initialSetup;
        });
    }
    onMeDataEmitted(data) {
        return __awaiter(this, void 0, void 0, function* () {
            switch (data.type) {
                case IMeInterfaces_1.EmittedDataType.AudioMutedChanged:
                    this.onMeAudioMutedChanged(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.BlurStarted:
                    this.onMeBlurStarted(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.BlurStopped:
                    this.onMeBlurStopped(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.MaskStarted:
                    this.onMeMaskStarted(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.MaskStopped:
                    this.onMeMaskStopped(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.NewCameraPublisher:
                    this.onMeNewCameraPublisher(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.NewIceCandidate:
                    this.onMeNewIceCandidate(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.NewScreenPublisher:
                    this.onMeNewScreenPublisher(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.CameraPublisherOffer:
                    this.onMeCameraPublisherOffer(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.ScreenPublisherOffer:
                    this.onMeScreenPublisherOffer(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.RemovePublisher:
                    this.onMeRemovePublisher(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.VideoHiddenChanged:
                    this.onMeVideoHiddenChanged(data.data);
                    break;
                case IMeInterfaces_1.EmittedDataType.Volume:
                    this.onMeVolume(data.data);
                    break;
                default:
                    this._logger.error('Received unknown data from Me: ', data);
                    break;
            }
        });
    }
    onMeAudioMutedChanged(params) {
        this._signaler.send('publisher-status', {
            id: params.publisherId,
            isAudioMuted: params.muted
        });
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.AudioMutedChanged,
            data: {
                publisherId: params.publisherId,
                muted: params.muted
            }
        });
    }
    onMeBlurStarted(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.BlurStarted,
            data: params
        });
    }
    onMeBlurStopped(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.BlurStopped,
            data: params
        });
    }
    onMeMaskStarted(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.MaskStarted,
            data: params
        });
    }
    onMeMaskStopped(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.MaskStopped,
            data: params
        });
    }
    onMeNewCameraPublisher(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.NewPublisher,
            data: {
                id: params.id,
                streamKind: StreamKind_1.StreamKind.Camera,
                stream: params.stream,
                isAudioMuted: params.isAudioMuted,
                isVideoHidden: params.isVideoHidden,
                participantId: params.participantId,
                isBlurActive: false
            }
        });
    }
    onMeNewIceCandidate(params) {
        const newIceCandidate = {
            id: params.publisherId,
            candidate: params.candidate
        };
        this._logger.debug('Sending publisher new-ice-candidate: ', newIceCandidate);
        this._signaler.send('new-ice-candidate', newIceCandidate);
    }
    onMeNewScreenPublisher(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.NewPublisher,
            data: {
                id: params.id,
                streamKind: StreamKind_1.StreamKind.Screen,
                stream: params.stream,
                participantId: params.participantId,
            }
        });
    }
    onMeCameraPublisherOffer(params) {
        const publish = {
            id: params.id,
            isAudioMuted: params.isAudioMuted,
            isVideoHidden: params.isVideoHidden,
            offer: params.offer,
            streamKind: StreamKind_1.StreamKind.Camera
        };
        this._logger.debug('Sending publish: ', publish);
        this._signaler.send('publish', publish);
    }
    onMeScreenPublisherOffer(params) {
        const publish = {
            id: params.id,
            offer: params.offer,
            streamKind: StreamKind_1.StreamKind.Screen
        };
        this._logger.debug('Sending publish: ', publish);
        this._signaler.send('publish', publish);
    }
    onMeRemovePublisher(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.RemovePublisher,
            data: params
        });
        this._signaler.send('unpublish', params);
    }
    onMeVideoHiddenChanged(params) {
        this._signaler.send('publisher-status', {
            id: params.publisherId,
            isVideoHidden: params.hidden
        });
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.VideoHiddenChanged,
            data: {
                publisherId: params.publisherId,
                hidden: params.hidden
            }
        });
    }
    onMeVolume(params) {
        this._emitter.next({
            type: SessionEmittedDataType_1.SessionEmittedDataType.MyVolume,
            data: params
        });
    }
    /**
     * Replaces own audio track by specifying a different device (microphone) Id.\
     * \
     * See also: [MediaDeviceInfo](https://developer.mozilla.org/en-US/docs/Web/API/MediaDeviceInfo)
     */
    replaceAudioTrack(publisherId, deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._me.replaceAudioTrack(publisherId, deviceId);
        });
    }
    /**
    * Replaces own video track by specifying a different device (camera) Id.\
    * \
    * See also: [MediaDeviceInfo](https://developer.mozilla.org/en-US/docs/Web/API/MediaDeviceInfo)
    */
    replaceVideoTrack(publisherId, deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._me.replaceVideoTrack(publisherId, deviceId);
        });
    }
    /**
     * Sends a message to session participants.
     */
    sendMessage(text, params) {
        return __awaiter(this, void 0, void 0, function* () {
            const message = { id: (0, uuid_1.v4)(), text: text };
            const result = { id: message.id };
            if (params === null || params === void 0 ? void 0 : params.attachments) {
                switch (params.attachments.type) {
                    case ISignaler_1.AttachmentType.File:
                        const files = params.attachments.data;
                        if (files.some(f => f.size > 15728640)) {
                            throw new Error('Max allowed file size is 15MB.');
                        }
                        message.attachments = {
                            type: ISignaler_1.AttachmentType.File,
                            data: yield Promise.all(files.map((f) => __awaiter(this, void 0, void 0, function* () {
                                return {
                                    id: yield this._fileBox.add(f, message.id),
                                    name: f.name,
                                    size: f.size,
                                    chunks: Math.ceil(f.size / this._fileBox.chuckSize)
                                };
                            })))
                        };
                        result.uploadingFiles = message.attachments.data.map(f => {
                            return {
                                id: f.id,
                                name: f.name,
                                size: f.size
                            };
                        });
                        break;
                }
            }
            if (params === null || params === void 0 ? void 0 : params.mentions) {
                message.mentions = params.mentions;
            }
            if (params === null || params === void 0 ? void 0 : params.replyId) {
                message.replyId = params.replyId;
            }
            this._signaler.send('new-message', message);
            return result;
        });
    }
    /**
     * Informs other participants if you are typing or not.
     */
    sendTyping(typing) {
        this._signaler.send('typing', {
            typing: typing
        });
    }
    /**
   * Apply blur background effect to own video track.
   */
    applyBlur() {
        this._me.applyBlur();
    }
    /**
   * Remove blur background effect to own video track.
   */
    removeBlur() {
        this._me.removeBlur();
    }
    /**
   * Apply mask effect (change background with a custom image) to own video track.
     * @param imageUrl The URL of the image to use as a background. If the URL domain of the image is different from the web application domain, the server providing the image must add the following response header "Access-Control-Allow-Origin: *".
     * @param metadata Data that will return with the [[MaskStarted]] event. Useful, for example, to keep track of the image chosen as a background.
   */
    applyMask(imageUrl, metadata = null) {
        this._me.applyMask(imageUrl, metadata);
    }
    /**
   * Remove mask effect from own video track.
   */
    removeMask() {
        this._me.removeMask();
    }
    /**
     * Toggles screensharing.
     */
    toggleScreensharing() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._mode === Mode_1.Mode.AudioAndVideo || this._mode === Mode_1.Mode.VideoOnly) {
                yield this._me.toggleScreensharing();
            }
            else {
                this._logger.warn('It is not possible to start a screen sharing during a non-video session.');
            }
        });
    }
    /**
     * Toggles audio (microphone).\
     * This can be used to mute/unmute own microphone.
     */
    toggleAudio(publisherId) {
        this._me.toggleAudio(publisherId);
    }
    /**
     * Toggles video (camera).\
     * This can be used to show/hide own camera.
     */
    toggleVideo(publisherId) {
        this._me.toggleVideo(publisherId);
    }
}
exports.Session = Session;
