"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BackgroundEngine = void 0;
require("@tensorflow/tfjs-backend-webgl");
const rxjs_1 = require("rxjs");
const BlurFliter_1 = require("./TensorFlow/BlurFliter");
const MaskFilter_1 = require("./TensorFlow/MaskFilter");
class BackgroundEngine {
    constructor(stream) {
        this._emitter = new rxjs_1.Subject();
        this._isActionPending = false;
        this.initializeFilters(stream);
    }
    get emitter() {
        return this._emitter.asObservable();
    }
    initializeFilters(stream) {
        this._blurFilter = new BlurFliter_1.BlurFilter(stream);
        this._blurFilterSubscription = this._blurFilter.emitter.subscribe((event) => this.onFilterEvent(event));
        this._maskFilter = new MaskFilter_1.MaskFilter(stream);
        this._maskFilterSubscription = this._maskFilter.emitter.subscribe((event) => this.onFilterEvent(event));
    }
    onFilterEvent(event) {
        this._emitter.next(event);
        this._isActionPending = false;
    }
    dispose() {
        this.removeActiveFilters();
        this._blurFilterSubscription.unsubscribe();
        this._maskFilterSubscription.unsubscribe();
    }
    removeActiveFilters() {
        if (this._blurFilter.isActive) {
            this._blurFilter.removeBlur();
        }
        if (this._maskFilter.isActive) {
            this._maskFilter.removeMask();
        }
    }
    applyBlur() {
        if (this._isActionPending === true) {
            console.warn('Another action is pending. Please try again later.');
            return;
        }
        this._isActionPending = true;
        this.removeActiveFilters();
        this._blurFilter.applyBlur();
    }
    removeBlur() {
        if (this._isActionPending === true) {
            console.warn('Another action is pending. Please try again later.');
            return;
        }
        this._isActionPending = true;
        this._blurFilter.removeBlur();
    }
    applyMask(imageUrl, metadata) {
        if (!imageUrl) {
            throw new Error(`The image URL provided is not valid: ${imageUrl}`);
        }
        if (this._isActionPending === true) {
            console.warn('Another action is pending. Please try again later.');
            return;
        }
        this._isActionPending = true;
        this.removeActiveFilters();
        this._maskFilter.applyMask(imageUrl, metadata);
    }
    removeMask() {
        if (this._isActionPending === true) {
            console.warn('Another action is pending. Please try again later.');
            return;
        }
        this._isActionPending = true;
        this._maskFilter.removeMask();
    }
}
exports.BackgroundEngine = BackgroundEngine;
