"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlurFilter = void 0;
const rxjs_1 = require("rxjs");
const IBackgroundEngine_1 = require("../interfaces/IBackgroundEngine");
const bodyPix = require('@tensorflow-models/body-pix');
class BlurFilter {
    constructor(stream) {
        this.SEGMENTATION_THRESHOLD = 0.8;
        this.INTERNAL_RESOLUTION = 0.5;
        this._emitter = new rxjs_1.Subject();
        this._originalVideoStream = new MediaStream();
        this.blurTrack = () => __awaiter(this, void 0, void 0, function* () {
            this._originalVideoElement.removeEventListener('loadeddata', this.blurTrack);
            const toCanvas = () => __awaiter(this, void 0, void 0, function* () {
                if (this._originalVideoElement.paused) {
                    return;
                }
                if (!this._isActive) {
                    this._isActive = true;
                    this._emitter.next({ type: IBackgroundEngine_1.BackgroundEventType.BlurStarted, data: this._stream.getVideoTracks()[0] });
                }
                // Get person segmentation from an image or a video.
                // In case of video, only one frame is used.
                const segmentation = yield this._net.segmentPerson(this._originalVideoElement, {
                    flipHorizontal: false,
                    internalResolution: this.INTERNAL_RESOLUTION,
                    segmentationThreshold: this.SEGMENTATION_THRESHOLD
                });
                // How many pixels in the background blend into each other (0-20).
                const backgroundBlurAmount = 10;
                // How many pixels to blur on the edge between the person and the background by (0-20).
                const edgeBlurAmount = 10;
                // If the output should be flipped horizontally.
                const flipHorizontal = false;
                // Draw the image with the background blurred onto the canvas.
                bodyPix.drawBokehEffect(this._outputCanvasElement, this._originalVideoElement, segmentation, backgroundBlurAmount, edgeBlurAmount, flipHorizontal);
                // Loop the animation frame.
                this._animationFrameId = requestAnimationFrame(toCanvas);
            });
            this._animationFrameId = requestAnimationFrame(toCanvas);
        });
        this._stream = stream;
        bodyPix.load({
            architecture: 'MobileNetV1',
            outputStride: 16,
            multiplier: 0.50,
            quantBytes: 2
        }).then(net => {
            this._net = net;
        });
    }
    get emitter() {
        return this._emitter.asObservable();
    }
    get isActive() {
        return this._isActive;
    }
    applyBlur() {
        if (!this._net) {
            throw new Error('You can\'t blur video yet. Please try again later.');
        }
        this.initializeElements();
        this.replaceCameraTrackWithCanvasTrack();
    }
    initializeElements() {
        const track = this._stream.getVideoTracks()[0];
        this._frameWidth = track.getSettings().width;
        this._frameHeight = track.getSettings().height;
        this._outputCanvasElement = document.createElement('canvas');
        this._outputCanvasElement.height = this._frameHeight;
        this._outputCanvasElement.width = this._frameWidth;
        this._originalVideoElement = document.createElement('video');
        this._originalVideoElement.height = this._frameHeight;
        this._originalVideoElement.width = this._frameWidth;
        this._originalVideoElement.autoplay = true;
        this._originalVideoStream.addTrack(this._stream.getVideoTracks()[0]);
        this._originalVideoElement.addEventListener('loadeddata', this.blurTrack);
        this._originalVideoElement.srcObject = this._originalVideoStream;
    }
    replaceCameraTrackWithCanvasTrack() {
        this._stream.removeTrack(this._stream.getVideoTracks()[0]);
        this._outputCanvasElement.getContext('2d'); // Fix Firefox initialization exception. See: https://stackoverflow.com/questions/63140354/firefox-gives-irregular-initialization-error-when-trying-to-use-navigator-mediad
        const canvasStream = this._outputCanvasElement.captureStream(10);
        this._stream.addTrack(canvasStream.getVideoTracks()[0]);
    }
    removeBlur() {
        this.restoreCameraStream();
        this._isActive = false;
        this._emitter.next({ type: IBackgroundEngine_1.BackgroundEventType.BlurStopped });
    }
    restoreCameraStream() {
        cancelAnimationFrame(this._animationFrameId);
        const track = this._originalVideoStream.getVideoTracks()[0];
        this._originalVideoStream.removeTrack(track);
        this._originalVideoElement.srcObject = null;
        this._originalVideoElement.remove();
        this._outputCanvasElement.remove();
        this._stream.removeTrack(this._stream.getVideoTracks()[0]);
        this._stream.addTrack(track);
    }
}
exports.BlurFilter = BlurFilter;
