"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MaskFilter = void 0;
const rxjs_1 = require("rxjs");
const IBackgroundEngine_1 = require("../interfaces/IBackgroundEngine");
const bodyPix = require('@tensorflow-models/body-pix');
class MaskFilter {
    constructor(stream) {
        this.SEGMENTATION_THRESHOLD = 0.8;
        this.INTERNAL_RESOLUTION = 0.5;
        this._emitter = new rxjs_1.Subject();
        this._originalVideoStream = new MediaStream();
        this.onBackgroundImageLoad = () => {
            this._backgroundImage.removeEventListener('load', this.onBackgroundImageLoad);
            const backgroundCanvas = document.createElement('canvas');
            backgroundCanvas.height = this._frameHeight;
            backgroundCanvas.width = this._frameWidth;
            const backgroundContext = backgroundCanvas.getContext('2d');
            backgroundContext.drawImage(this._backgroundImage, 0, 0, this._frameWidth, this._frameHeight);
            this._backgroundImageData = backgroundContext.getImageData(0, 0, this._frameWidth, this._frameHeight);
            backgroundCanvas.remove();
            this._backgroundImage.remove();
            this._originalVideoElement.addEventListener('loadeddata', this.maskTrack);
            this._originalVideoElement.srcObject = this._originalVideoStream;
        };
        this.maskTrack = () => __awaiter(this, void 0, void 0, function* () {
            this._originalVideoElement.removeEventListener('loadeddata', this.maskTrack);
            const toCanvas = () => __awaiter(this, void 0, void 0, function* () {
                if (this._originalVideoElement.paused) {
                    return;
                }
                if (!this._isActive) {
                    this._isActive = true;
                    this._emitter.next({
                        type: IBackgroundEngine_1.BackgroundEventType.MaskStarted,
                        data: {
                            track: this._stream.getVideoTracks()[0],
                            metadata: this._metadata
                        }
                    });
                    this._metadata = null;
                }
                // Get person segmentation from an image or a video.
                // In case of video, only one frame is used.
                const segmentation = yield this._net.segmentPerson(this._originalVideoElement, {
                    flipHorizontal: false,
                    internalResolution: this.INTERNAL_RESOLUTION,
                    segmentationThreshold: this.SEGMENTATION_THRESHOLD
                });
                const cameraCanvas = document.createElement('canvas');
                cameraCanvas.height = this._frameHeight;
                cameraCanvas.width = this._frameWidth;
                const cameraContext = cameraCanvas.getContext('2d');
                cameraContext.drawImage(this._originalVideoElement, 0, 0);
                const cameraImageData = cameraContext.getImageData(0, 0, this._frameWidth, this._frameHeight);
                cameraCanvas.remove();
                const backgroundImageData = new ImageData(new Uint8ClampedArray(this._backgroundImageData.data), this._backgroundImageData.width, this._backgroundImageData.height);
                for (let i = 0; i < segmentation.data.length; i++) {
                    if (segmentation.data[i] === 1) {
                        backgroundImageData.data[i * 4] = cameraImageData.data[i * 4];
                        backgroundImageData.data[(i * 4) + 1] = cameraImageData.data[(i * 4) + 1];
                        backgroundImageData.data[(i * 4) + 2] = cameraImageData.data[(i * 4) + 2];
                        backgroundImageData.data[(i * 4) + 3] = cameraImageData.data[(i * 4) + 3];
                    }
                }
                const outputCanvas = this._outputCanvasElement.getContext('2d');
                outputCanvas.putImageData(backgroundImageData, 0, 0);
                // Loop the animation frame.
                this._animationFrameId = requestAnimationFrame(toCanvas);
            });
            this._animationFrameId = requestAnimationFrame(toCanvas);
        });
        this._stream = stream;
        bodyPix.load({
            architecture: 'MobileNetV1',
            outputStride: 16,
            multiplier: 0.50,
            quantBytes: 2
        }).then(net => {
            this._net = net;
        });
    }
    get emitter() {
        return this._emitter.asObservable();
    }
    get isActive() {
        return this._isActive;
    }
    applyMask(imageUrl, metadata) {
        if (this._isActive) {
            throw new Error('A mask is already active. Please remove the active mask before applying a new one.');
        }
        this._metadata = metadata;
        this.initializeElements(imageUrl);
        this.replaceCameraTrackWithCanvasTrack();
    }
    initializeElements(imageUrl) {
        const track = this._stream.getVideoTracks()[0];
        this._frameWidth = track.getSettings().width;
        this._frameHeight = track.getSettings().height;
        this._outputCanvasElement = document.createElement('canvas');
        this._outputCanvasElement.height = this._frameHeight;
        this._outputCanvasElement.width = this._frameWidth;
        this._originalVideoElement = document.createElement('video');
        this._originalVideoElement.height = this._frameHeight;
        this._originalVideoElement.width = this._frameWidth;
        this._originalVideoElement.autoplay = true;
        this._originalVideoStream.addTrack(this._stream.getVideoTracks()[0]);
        this._backgroundImage = document.createElement('img');
        // This will ignore CORS when background image is hosted on a different domain.
        // To make this work, the server application must add the following response header: Access-Control-Allow-Origin: "*"
        // See: https://stackoverflow.com/questions/22097747/how-to-fix-getimagedata-error-the-canvas-has-been-tainted-by-cross-origin-data
        this._backgroundImage.crossOrigin = "Anonymous";
        this._backgroundImage.height = this._frameHeight;
        this._backgroundImage.width = this._frameWidth;
        this._backgroundImage.addEventListener('load', this.onBackgroundImageLoad);
        this._backgroundImage.src = imageUrl;
    }
    replaceCameraTrackWithCanvasTrack() {
        this._stream.removeTrack(this._stream.getVideoTracks()[0]);
        this._outputCanvasElement.getContext('2d'); // Fix Firefox initialization exception. See: https://stackoverflow.com/questions/63140354/firefox-gives-irregular-initialization-error-when-trying-to-use-navigator-mediad
        const canvasStream = this._outputCanvasElement.captureStream(10);
        this._stream.addTrack(canvasStream.getVideoTracks()[0]);
    }
    removeMask() {
        if (!this._isActive) {
            return;
        }
        this.restoreCameraStream();
        this._isActive = false;
        this._emitter.next({ type: IBackgroundEngine_1.BackgroundEventType.MaskStopped });
    }
    restoreCameraStream() {
        cancelAnimationFrame(this._animationFrameId);
        const track = this._originalVideoStream.getVideoTracks()[0];
        this._originalVideoStream.removeTrack(track);
        this._originalVideoElement.srcObject = null;
        this._originalVideoElement.remove();
        this._outputCanvasElement.remove();
        this._stream.removeTrack(this._stream.getVideoTracks()[0]);
        this._stream.addTrack(track);
    }
}
exports.MaskFilter = MaskFilter;
