import { Subscription } from "rxjs";
import { AttachmentType } from "../enums/AttachmentType";
import { ErrorCode } from "../enums/ErrorCode";
import { Mode } from "../enums/Mode";
import { Reason } from "../enums/Reason";
import { SessionEmittedDataType } from "../enums/SessionEmittedDataType";
import { StreamKind } from "../enums/StreamKind";
import { IInitialSetup } from "./IPublisherInterfaces";
/**
 * An attachments list sent with a message.
 * \
 * See also: [[INewMessage]].
 */
export interface IAttachments {
    /**
     * The type of the attachments.
     */
    type: AttachmentType;
    /**
     * The attachments list.
     */
    data: IFileData[];
}
/**
 * An object identifying generic file data, used when [[AttachmentType]] is of type File.
 */
export interface IFileData {
    /**
     * The Id of the file.
     */
    id: string;
    /**
     * The name of the file.
     */
    name: string;
    /**
     * The size of the file in bytes.
     */
    size: number;
    /**
     * The URL to download the file.
     */
    url: string;
}
export interface IParticipantSubscription {
    participantId: string;
    subscription: Subscription;
}
/**
 * Data emitted by session with [[AudioMutedChanged]] type.
 */
export interface IAudioMutedChanged {
    /**
     * The Id of the publisher whose audio state has changed.
     */
    publisherId: string;
    /**
     * The audio new state.
     */
    muted: boolean;
}
/**
 * Data emitted by session with [[BlurStarted]] type.
 */
export interface IBlurStarted {
    /**
     * The Id of the publisher whose video track has been blurred.
     */
    publisherId: string;
}
/**
 * Data emitted by session with [[BlurStopped]] type.
 */
export interface IBlurStopped {
    /**
     * The Id of the publisher whose blur effect has been removed from the video track.
     */
    publisherId: string;
}
/**
 * Data emitted by session with [[MaskStarted]] type.
 */
export interface IMaskStarted {
    /**
     * The Id of the publisher whose video track has been masked.
     */
    publisherId: string;
    /**
     * Data passed to [[applyMask]] method.
     */
    metadata: any;
}
/**
 * Data emitted by session with [[MaskStopped]] type.
 */
export interface IMaskStopped {
    /**
     * The Id of the publisher whose mask effect has been removed from the video track.
     */
    publisherId: string;
}
/**
 * Data emitted by session with [[DeleteMessage]] type.
 */
export interface IDeleteMessage {
    /**
     * The Id of the message has been deleted.
     */
    id: string;
}
/**
 * Data emitted by session with [[EditMessage]] type.
 */
export interface IEditMessage {
    /**
     * The Id of the message has been edited.
     */
    id: string;
    /**
     * The new message text.
     */
    text: string;
    /**
     * The Id list of the participant this message is mentioning.
     */
    mentions?: IMention[];
}
/**
 * Data emitted by session with [[FileProgress]] type.
 */
export interface IFileProgress {
    /**
     * The Id of the file.
     */
    id: string;
    /**
     * The Id of the message the file belongs to.
     */
    messageId: string;
    /**
     * The progress percentage of the upload.
     */
    progress: number;
}
/**
 * Data emitted by session with [[Me]] and [[NewParticipant]] type.
 */
export interface INewParticipant {
    /**
     * The Id of the new participant.
     */
    id: string;
    /**
     * The nickname of the new participant.
     */
    nickname: string;
}
/**
 * Data emitted by session with [[NewPublisher]] type.
 */
export interface INewPublisher {
    /**
     * The Id of the new publisher.
     */
    id: string;
    /**
     * The state of the publisher audio.
     */
    isAudioMuted: boolean;
    /**
     * The kind of the publisher video.
     */
    streamKind: StreamKind;
    /**
     * The Id of the participant this publisher belongs to.
     */
    participantId: string;
    /**
     * The media stream of the publisher.\
     * \
     * See also: [MediaStream](https://developer.mozilla.org/en-US/docs/Web/API/MediaStream).
     */
    stream: MediaStream;
    /**
     * The state of the publisher video.
     */
    isVideoHidden: boolean;
    /**
     * Indicates whether the background blur effect is active.
     */
    isBlurActive: boolean;
    /**
     * Indicates whether the background mask effect is active.
     */
    isMaskActive: boolean;
}
/**
 * Data emitted by session with [[NewSubscriber]] type.
 */
export interface INewSubscriber {
    /**
     * The Id of the new subscriber.
     */
    id: string;
    /**
     * The Id of the participant this subscriber belongs to.
     */
    participantId: string;
    /**
     * The media stream of the subscriber.\
     * \
     * See also: [MediaStream](https://developer.mozilla.org/en-US/docs/Web/API/MediaStream).
     */
    stream: MediaStream;
    /**
     * The state of stream audio track.
     */
    isAudioMuted: boolean;
    /**
     * The state of stream video track.
     */
    isVideoHidden: boolean;
    /**
     * The kind of the subscriber video.
     */
    streamKind: StreamKind;
}
/**
 * Data emitted by session with [[NewMessage]] type.
 */
export interface INewMessage {
    /**
     * The Id of the message.
     */
    id: string;
    /**
     * The Id of the participant this message belongs to.
     */
    participantId: string;
    /**
     * The nickname of the sender.
     */
    nickname: string;
    /**
     * The text of the message.
     */
    text?: string;
    /**
     * A list of files attached to the message.
     */
    attachments?: IAttachments;
    /**
     * The Id of the message this message is replying to.
     */
    replyId?: string;
    /**
     * The Id list of the participant this message is mentioning.
     */
    mentions?: IMention[];
    /**
     * The date of the message.
     */
    sentAt: Date;
}
/**
 * Data emitted by session with [[RecStarted]] type.
 */
export interface IRecStarted {
    /**
     * The Id of the recording that has just started.
     */
    id: string;
}
/**
 * Data emitted by session with [[RemoveParticipant]] type.
 */
export interface IRemoveParticipant {
    /**
     * The Id of the participant that has left the session.
     */
    id: string;
}
/**
 * Data emitted by session with [[RemovePublisher]] type.
 */
export interface IRemovePublisher {
    /**
     * The Id of the own publisher removed.
     */
    id: string;
}
/**
 * Data emitted by session with [[RemoveSubscriber]] type.
 */
export interface IRemoveSubscriber {
    /**
     * The Id of the removed subscriber.
     */
    id: string;
    /**
     * The Id of the participant the removed subscriber belongs to.
     */
    participantId: string;
}
/**
 * Data emitted by session with [[Error]] type.
 */
export interface IError {
    /**
     * The error code.\
     * \
     * See also [[ErrorCode]].
     */
    code: ErrorCode;
    /**
     * A message which describes the error.
     */
    message: string;
    /**
     * Error specific data.
     */
    metadata?: any;
}
/**
 * Data emitted by session with [[SessionEnded]] type.
 */
export interface ISessionEnded {
    reason: {
        /**
         * The reason the meeting ended.
         */
        code: Reason;
        /**
         * A description, in English, of the reason the meeting ended.
         */
        description: string;
    };
}
/**
 * Data emitted by session with [[SubscriberStatus]] type.
 */
export interface ISubscriberStatus {
    /**
     * The Id of the subscriber.
     */
    id: string;
    /**
     * The Id of the participant the subscriber belongs to.
     */
    participantId: string;
    /**
     * The state of audio track.
     */
    isAudioMuted: boolean;
    /**
     * The state of video track.
     */
    isVideoHidden: boolean;
}
/**
 * Data emitted by session with [[Typing]] type.
 */
export interface ITyping {
    /**
     * The Id of the participant has started/stopped typing.
     */
    participantId: string;
    /**
     * True if the participant is typing, false otherwise.
     */
    typing: boolean;
}
/**
 * Data emitted by session with [[VideoHiddenChanged]] type.
 */
export interface IVideoHiddenChanged {
    /**
     * The Id of the publisher whose video state has changed.
     */
    publisherId: string;
    /**
     * The video new state.
     */
    hidden: boolean;
}
/**
 * Data emitted by session with [[MyVolume]] type.
 */
export interface IMyVolume {
    /**
     * The Id of the publisher.
     */
    publisherId: string;
    /**
     * The current volume value of the audio track.
     */
    volume: number;
}
/**
 * Data emitted by session with [[ParticipantVolume]] type.
 */
export interface IParticipantVolume {
    /**
     * The Id of the participant.
     */
    participantId: string;
    /**
     * The Id of the subscriber.
     */
    subscriberId: string;
    /**
     * The current volume value of the audio track.
     */
    volume: number;
}
/**
 * Data emitted by session with [[WebSocketConnected]] type.
 */
export interface IWebSocketConnected {
    /**
     * The Id of the websocket.
     */
    id: string;
}
/**
 * Data emitted by session with [[Welcome]] type.
 */
export interface IWelcome {
    /**
     * The Id of the session.
     */
    id: string;
    /**
     * The topic of the session.
     */
    topic: string;
    /**
     * The capabilities of the session, based on the subscribed plan.
     */
    capabilities: {
        /**
         * Indicates whether the session supports audio.
         */
        audio: boolean;
        /**
         * Indicates whether the session supports video.
         */
        video: boolean;
        /**
         * Indicates whether the session supports chat messages.
         */
        chat: boolean;
    };
    /**
     * The Id of the recording if the recording is active when joining the session, null otherwise.
     */
    recordingId: string;
    /**
     * Indicates whether the display of a satisfaction feedback is required at the end of the session.
     */
    feedback: boolean;
    /**
     * The URL to be used for guest participants redirection when the meeting ends.
     */
    guestRedirectionUrl?: string;
}
export interface ISessionEmittedData {
    type: SessionEmittedDataType;
    data?: string | IAudioMutedChanged | IBlurStarted | IBlurStopped | IMaskStarted | IMaskStopped | IMyVolume | INewParticipant | INewPublisher | INewSubscriber | INewMessage | IParticipantVolume | IRecStarted | IRemoveParticipant | IRemovePublisher | IRemoveSubscriber | ISessionEnded | IError | ITyping | IVideoHiddenChanged | IWelcome;
}
/**
 * Configuration data for meeting initialization.\
 * \
 * See also: [[joinMeeting]].
 */
export interface IMeetingConfig {
    /**
     * The initial camera setup of the own publisher when joining the meeting.
     */
    publisherInitialSetup?: IInitialSetup;
    /**
     * The Id of the meeting to join.
     */
    meetingId: string;
    /**
     * The nickname used by other participants to identify you.
     */
    nickname: string;
    /**
     * The streams mode of your own session.\
     * This option cannot be changed during the session.
     */
    mode?: Mode;
    /**
     * The url of the web socket to connect to.\
     * It is composed by the concatenation of Vidaoo's backend base url and the string '/meetings'.\
     * Example: https://vidaoo-url.com/meetings
     */
    webSocketUrl: string;
}
/**
 * An object that represents a mention within the message text.
 */
export interface IMention {
    /**
     * An index that identifies the beginning of the mention within the text.
     */
    start: number;
    /**
     * The length of the mention.
     */
    length: number;
    /**
     * The Id of the participant that has been mentioned.
     */
    participantId: string;
}
/**
 * Optional message parameters.
 */
export interface IMessageParams {
    /**
     * The Id of the message this message is replying to.
     */
    replyId?: string;
    /**
     * The Id list of the participant this message is mentioning.
     */
    mentions?: IMention[];
    /**
     * A list of files attached to the message.
     */
    attachments?: {
        type: AttachmentType;
        data: File[];
    };
}
/**
 * A set of data concerning the message that has been sent.
 */
export interface ISendMessageResult {
    /**
     * The Id of the message.
     */
    id: string;
    /**
     * Data regarding any attached files.
     */
    uploadingFiles?: IUploadingFile[];
}
/**
 * An object that represents a file that is about to be uploaded.
 */
export interface IUploadingFile {
    /**
     * The Id of the file.
     */
    id: string;
    /**
     * The name of the file.
     */
    name: string;
    /**
     * The size of the file, in bytes.
     */
    size: number;
}
